'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = diffTree;
function indexOf(nodes, node, isSame, nodeIndex) {
  var len = nodes.length;
  for (var i = 0; i < len; i++) {
    if (isSame(node, nodes[i], nodeIndex, i)) {
      return i;
    }
  }
  return -1;
}

function nativeShould(a, b) {
  return a === b;
}

function sortByIndex(a, b) {
  if (a.currentIndex === b.currentIndex) {
    return 0;
  }
  return a.currentIndex > b.currentIndex ? -1 : 1;
}

// diff by level
function diff(currentNodes, nextNodes) {
  var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
  var internal = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];
  var _options$shouldUpdate = options.shouldUpdate;
  var shouldUpdate = _options$shouldUpdate === undefined ? nativeShould : _options$shouldUpdate;
  var _options$childrenKey = options.childrenKey;
  var childrenKey = _options$childrenKey === undefined ? 'children' : _options$childrenKey;
  var _internal$path = internal.path;
  var path = _internal$path === undefined ? [] : _internal$path;
  var parentNode = internal.parentNode;

  var insertQueue = [];
  var updateQueue = [];
  var removeQueue = [];
  var lastIndex = 0;
  var tmp = void 0;
  nextNodes.forEach(function (nextNode, nextIndex) {
    var currentIndex = indexOf(currentNodes, nextNode, shouldUpdate, nextIndex);
    if (currentIndex !== -1) {
      var currentNode = currentNodes[currentIndex];
      updateQueue.push({
        type: 'update',
        currentNode: currentNode,
        nextNode: nextNode,
        parentNode: parentNode,
        path: path.concat(currentIndex)
      });
      if (currentIndex < lastIndex) {
        tmp = {
          type: 'move',
          currentNode: currentNode,
          nextNode: nextNode,
          parentNode: parentNode,
          currentIndex: currentIndex,
          path: path.concat(currentIndex),
          toPath: path.concat(nextIndex)
        };
        insertQueue.push(tmp);
        removeQueue.push(tmp);
      }
      lastIndex = Math.max(currentIndex, lastIndex);
    } else {
      insertQueue.push({
        type: 'new',
        nextNode: nextNode,
        parentNode: parentNode,
        toPath: path.concat(nextIndex)
      });
    }
  });

  currentNodes.forEach(function (currentNode, currentIndex) {
    var nextIndex = indexOf(nextNodes, currentNode, shouldUpdate, currentIndex);
    if (nextIndex === -1) {
      removeQueue.push({
        type: 'remove',
        currentNode: currentNode,
        parentNode: parentNode,
        currentIndex: currentIndex,
        path: path.concat(currentIndex)
      });
    }
  });

  removeQueue.sort(sortByIndex);

  if (childrenKey) {
    updateQueue.concat().forEach(function (o) {
      var currentChildren = o.currentNode[childrenKey] || [];
      var nextChildren = o.nextNode[childrenKey] || [];
      // bottom up
      var ret = diff(currentChildren, nextChildren, options, {
        path: o.path,
        parentNode: o.currentNode
      });
      insertQueue = ret.insertQueue.concat(insertQueue);
      updateQueue = ret.updateQueue.concat(updateQueue);
      removeQueue = ret.removeQueue.concat(removeQueue);
    });
  }

  return {
    insertQueue: insertQueue,
    updateQueue: updateQueue,
    removeQueue: removeQueue
  };
}

function diffTree(currentNodes, nextNodes, options) {
  return diff(currentNodes, nextNodes, options);
}
module.exports = exports['default'];